#!/usr/bin/env python
"""
Interactive wxmplot

 provides simple 'plot()' and 'imshow()' functions to python interpreter

 plot(x, y):  display a simple XY line plot to a wxmplot.PlotFrame

 imshow(array): display image of 2D array data on a wxmplot.ImageFrame

 set_theme(themename): set plotting theme ('light', 'dark', 'seaborn', ...).

"""

import time
import os
import sys
import atexit

import wx

from . import inputhook
from .plotframe import PlotFrame
from .imageframe import ImageFrame
from .stackedplotframe import StackedPlotFrame
from .config import Themes

IMG_DISPLAYS = {}
PLOT_DISPLAYS = {}
MAX_WINDOWS = 100
MAX_CURSHIST = 100
DEFAULT_THEME = 'light'

__all__ = ['wxapp', 'plot', 'newplot', 'imshow', 'get_wxapp', 'set_theme',
           'available_themes', 'get_plot_window', 'get_image_window',
           'update_trace', 'plot_setlimits', 'plot_text', 'plot_arrow',
           'plot_marker', 'plot_axhline', 'plot_axvline', 'hist',
           'contour', 'DEFAULT_THEME']

wxapp = None
def get_wxapp(redirect=False, clearSigInt=True):
    """get the wx App

    Args:
        redirect(bool): whether to redirect output that would otherwise
            be written to the Console [False]
        clearSigInt(bool): whether to clear interrupts of Ctrl-C [True]

    Returns:
        a wx.App instance


    """
    global wxapp
    if wxapp is None:
        wxapp = wx.GetApp()
        if wxapp is None:
            wxapp = wx.App(redirect=redirect, clearSigInt=clearSigInt)
    return wxapp

@atexit.register
def __wxmainloop__():
    """run wxApp mainloop, allowing widget interaction
    until all plotting and image image windows are closed.
    Note that this should not be necessary to run explicitly,
    as it is registered to run when Python exits
    """
    get_wxapp().MainLoop()


def set_theme(theme):
    """set plotting theme by name with a theme name

    Args:
        theme(str): name of theme


    Returns:
        None

    Notes:
      1. Example themese are:'light', 'dark', 'white-background', 'matplotlib',
      'seaborn', 'ggplot', 'bmh', 'fivethirtyeight'.
      2. See available_themes() for the list of available themes.
    """
    global DEFAULT_THEME
    if theme.lower() in Themes.keys():
        DEFAULT_THEME = theme.lower()
    else:
        raise ValueError("theme '%s' unavailable. use `availabale_themes()`" % theme)

def available_themes():
    """list of available theme

    Returns:
        list of theme names.

    Notes:
        As of this writing, the list is:

      'light', 'dark', 'white-background', 'matplotlib', 'seaborn',
      'ggplot', 'bmh', 'fivethirtyeight', 'grayscale', 'dark_background',
      'tableau-colorblind10', 'seaborn-bright', 'seaborn-colorblind',
      'seaborn-dark', 'seaborn-darkgrid', 'seaborn-dark-palette',
      'seaborn-deep', 'seaborn-notebook', 'seaborn-muted',
      'seaborn-pastel', 'seaborn-paper', 'seaborn-poster', 'seaborn-talk',
      'seaborn-ticks', 'seaborn-white', 'seaborn-whitegrid',
      'Solarize_Light2'


    """
    return [name for name in Themes.keys()]

class PlotDisplay(PlotFrame):
    def __init__(self, window=1, size=None, theme=None,
                 wintitle=None, **kws):
        get_wxapp()
        theme = DEFAULT_THEME if theme is None else theme

        if wintitle is None:
            wintitle   = 'Plot Window %i' % window

        PlotFrame.__init__(self, parent=None, size=size, title=wintitle,
                           output_title='plot', exit_callback=self.onExit,
                           theme=theme, **kws)

        if size is not None:
            self.SetSize(size)
        self.Show()
        self.Raise()
        self.panel.cursor_callback = self.onCursor
        self.panel.cursor_mode = 'zoom'
        self.window = int(window)
        self.cursor_hist = []
        if window not in PLOT_DISPLAYS:
            PLOT_DISPLAYS[window] = self


    def onExit(self, o, **kw):
        if self.window in PLOT_DISPLAYS:
            PLOT_DISPLAYS.pop(self.window)
        self.Destroy()

    def onCursor(self, x=None, y=None, **kw):
        self.cursor_hist.insert(0, (x, y, time.time()))
        if len(self.cursor_hist) > MAX_CURSHIST:
            self.cursor_hist = self.cursor_hist[:MAX_CURSHIST]

class ImageDisplay(ImageFrame):
    def __init__(self, window=1, size=None, theme=None,
                 wintitle=None, **kws):
        get_wxapp()
        if wintitle is None:
            wintitle   = 'Image Window %i' % window

        ImageFrame.__init__(self, parent=None, size=size, title=wintitle,
                            exit_callback=self.onExit, **kws)

        if size is not None:
            self.SetSize(size)
        self.Show()
        self.Raise()
        self.cursor_hist = []
        self.panel.cursor_callback = self.onCursor
        self.window = int(window)
        if self.window not in IMG_DISPLAYS:
            IMG_DISPLAYS[self.window] = self

    def onExit(self, o, **kw):
        if self.window in IMG_DISPLAYS:
            IMG_DISPLAYS.pop(self.window)
        self.Destroy()

    def onCursor(self,x=None, y=None, ix=None, iy=None, val=None, **kw):
        self.cursor_hist.insert(0, (x, y, ix, iy, time.time()))
        if len(self.cursor_hist) > MAX_CURSHIST:
            self.cursor_hist = self.cursor_hist[:MAX_CURSHIST]

def get_plot_window(win=1, size=None, wintitle=None, theme=None):
    """return a plot display

    Args:
        win (int): index of Plot Window (1 to 100)
        size (tuple): width, height in pixels of Plot Window
        wintitle(str): text for Window title [Plot Window N]
        theme (str): theme for Plot Window ['light']

    Returns:
        diplay, a wxmplot PlotFrame.

    Notes:
        this will either return the existing PlotFrame for the
        window index or create a new one.

    """
    win = max(1, min(MAX_WINDOWS, int(abs(win))))
    if win in PLOT_DISPLAYS:
        display = PLOT_DISPLAYS[win]
    else:
        display = PlotDisplay(window=win, size=size, theme=theme,
                              wintitle=wintitle)
    return display

def get_image_window(win=1, size=None, wintitle=None):
    """return an image display

    Args:
        win (int): index of Image Window (1 to 100)
        size (tuple): width, height in pixels of Image Window
        wintitle(str): text for Window title [Image Window N]

    Returns:
        diplay, a wxmplot ImageFrame.

    Notes:
        this will either return the existing ImageFrame for the
        window index or create a new one.
    """
    win = max(1, min(MAX_WINDOWS, int(abs(win))))
    if win in IMG_DISPLAYS:
        display = IMG_DISPLAYS[win]
    else:
        display = ImageDisplay(window=win, size=size,
                               wintitle=wintitle)

    return display

def plot(x,y, win=1, new=False, size=None, wintitle=None, theme=None, **kws):
    """plot(x, y, win=1, new=False, ...)

    Plot trace of x, y arrays in a PlotFrame

    Args:
        x (array-like):  ordinate values
        y (array-like):  abscissa values (x and y must be same size!)
        dy (array-like): array for error bars in y (must be same size as y!)
        new (bool): whether to start a new plot [False]
        win (int): index of Plot Window [1]
        label (str or None): label for this trace
        linewidth (float):  width of line joining points [from theme]
        color (str or None): color for trace (see note 1) [from theme]
        style (str or None): line style for joining points (see note 2)
        marker (str on None):  symbol to draw at each point (see note 3)
        markersize (float): size of marker
        zorder (float or None): zorder (depth) of this trace
        drawstyle (str or None): style for joining line segments (see note 4)
        xmin (float or None): minimum x value for plot range
        xmax (float or None): maximum x value for plot range
        ymin (float or None): minimum y value for plot range
        ymax (float or None): maximum y value for plot range
        use_dates (bool): whether to interpret x data as dates [False]
        side (str): side for y-axis ('left' or 'right') ['left']
        size (tuple or None): width, height in pixels of Plot Window
        wintitle (str or None): title for window frame
        theme (str on None): plotting theme to use
        title (str or None):  plot title
        xlabel (str or None): label for x-axis
        ylabel (str or None): label for y-axis
        y2label (str or None): label for y2-axis (that is, right-side axis)

        xlog_scale (bool): whether to show x-axis as log-scale [False]
        ylog_scale (bool): whether to show y-axis as log-scale [False]
        grid (bool): whether to draw background grid [True]
        show_legend (bool or None): whether to show legend [False]
        legend_loc (str): location for legend (see note 5) ['best']
        legend_on  (bool): whether legend is within the main axes [True]
        bgcolor (str on None): color of background plot area (from theme)
        framecolor (str on None): color of outer frame area (from theme)
        gridcolor (str or None):  color of grid (from theme)
        labelfontsize (float): size (pixels) of font for Labels (from theme)
        legendfontsize (float): size (pixels) of font for Legend (from theme)
        titlefontsize (float): size (pixels) of font for Title (from theme)
        axes_style (str): control what parts of axes are shown (see note 6)
        viewpad (float or None): percent of data range to expand view ranges
        delay_draw (boole): whether to delay drawing [False]

    Returns:
        plotter, a PlotFrame

    Notes:
        1. colors can be names such as 'red' or hex strings '#RRGGBB'.
           by default, they are set by the theme
        2. styles can be one of 'solid', 'short dashed', 'dash-dot',
           'dashed', 'dotted', 'long-dashed'.
        3. markers can be one of  'no symbol', 'o', '+', 'x', 'square',
           'diamond', 'thin diamond', '^', 'v', '>', '<', '|', '_',
           'hexagon', 'pentagon', 'tripod 1', or 'tripod 2'
        4. drawstyle can be one of 'default', 'steps-pre','steps-mid',
           or 'steps-post'.
        5. legend_loc can be one of 'best', 'upper right' , 'lower right',
           'center right', 'upper left', 'lower left',  'center left',
           'upper center', 'lower center', 'center'
        6. axis_style can be one of 'open', 'box', 'bottom'.

    """
    plotter = get_plot_window(win=win, size=size, wintitle=wintitle,
                             theme=theme)
    if plotter is None:
        return
    plotter.Raise()
    if new or plotter.panel.conf.ntrace == 0:
        plotter.plot(x, y, **kws)
    else:
        plotter.oplot(x, y, **kws)
    return plotter

def newplot(x, y, win=1, wintitle=None, **kws):
    """newplot(x, y, ...)

    Plot trace of x, y arrays in a PlotFrame, clearing any
    data currently shown in the PlotFrame.

    Notes:
        This is equivalent to
        plot(x, y, ...., new=True)

    """
    kws['new'] = True
    return plot(x, y, win=win, wintitle=wintitle, **kws)


def update_trace(x, y, trace=1, win=1, **kws):
    """
    update a plot trace with new data, avoiding complete redraw
    This can significantly descrease drawing time:

    Args:
        x (array-like):  ordinate values
        y (array-like):  abscissa values (x and y must be same size)
        trace (int): which plot trace to update [1]
        win (int): index of Plot Window [1]
        side (str): side for y-axis ('left' or 'right') ['left']

    """
    plotter = get_plot_window(win=win)
    if plotter is None:
        return
    plotter.Raise()
    trace -= 1 # wxmplot counts traces from 0
    plotter.panel.update_line(trace, x, y, draw=True, **kws)

def plot_setlimits(xmin=None, xmax=None, ymin=None, ymax=None, win=1):
    """set plot view limits for plot in window `win`

    Args:
        xmin (float): minimum x value
        ymax (float): maximum x value
        ymin (float): minimum y value
        ymax (float): maximum y value
        win (int): index of plot window
    """
    plotter = get_plot_window(win=win)
    if plotter is None:
        return
    plotter.panel.set_xylims((xmin, xmax, ymin, ymax))

def plot_text(text, x, y, win=1, rotation=None, ha='left', va='center',
              side='left', size=None, **kws):

    """plot_text(x, y, text, win=1, options)

    add text at x, y coordinates of a plot

    Args:
        text (str): text to draw
        x (float): x position of text
        y (float): y position of text
        win (int): index of plot window
        rotation (str or float):  text rotation. angle in degrees or 'vertical' or 'horizontal'
        ha (str):    horizontal alignment ('left', 'center', 'right')
        va (str)    vertical alignment ('top', 'center', 'bottom', 'baseline')
        side (str): which axis to use ('left' or 'right') for coordinates.


    """
    plotter = get_plot_window(win=win, size=size)
    if plotter is None:
        return
    plotter.Raise()
    plotter.add_text(text, x, y, rotation=rotation, ha=ha, va=va, **kws)

def plot_arrow(x1, y1, x2, y2, win=1, side='left',
               shape='full', color='black',
               width=0.00, head_width=0.05, head_length=0.25,
               size=None, **kws):
    """plot_arrow(x1, y1, x2, y2, win=1, **kws)

    draw arrow from x1, y1 to x2, y2.

    Args:
        x1 (float): starting x coordinate
        y1 (float): starting y coordinate
        x2 (float): ending x coordinate
        y2 (float): ending y coordinate
        side (str): which axis to use ('left' or 'right') for coordinates.
        shape (str):  arrow head shape ('full', 'left', 'right')
        color (str):  arrow color ('black')
        width (float):  width of arrow line (in points. default=0.0)
        head_width (float):  width of arrow head (in points. default=0.05)
        head_length (float):  length of arrow head (in points. default=0.25)
        overhang (float):   amount the arrow is swept back (in points. default=0)
        win (int): index of plot window

    """
    plotter = get_plot_window(win=win, size=size)
    if plotter is None:
        return
    plotter.Raise()
    plotter.add_arrow(x1, y1, x2, y2, side=side, shape=shape,
                      color=color, width=width, head_length=head_length,
                      head_width=head_width, **kws)

def plot_marker(x, y, marker='o', size=4, color='black', label='_nolegend_',
                win=1,  **kws):

    """plot_marker(x, y, marker='o', size=4, color='black')

    draw a marker at x, y

    Args:
        x (float): x coordinate for marker
        y float):  y coordinate
        marker (str): symbol to draw at each point (see Note 1)
        size (float): marker size [4]
        color (str): marker color  ['black']
        win (int): index of plot window

    Notes:
        1. marker can be one of ('+', 'o', 'x', 'square',
           'diamond', 'thin diamond', '^', 'v', '>', '<', '|', '_',
           'hexagon', 'pentagon', 'tripod 1', or 'tripod 2')

    """
    plotter = get_plot_window(win=win, size=None)
    if plotter is None:
        return
    plotter.Raise()
    plotter.oplot([x], [y], marker=marker, markersize=size, label=label,
                 color=color,  **kws)

def plot_axhline(y, xmin=0, xmax=1, win=1,
                  size=None, delay_draw=False,  **kws):
    """plot_axhline(y, xmin=None, ymin=None, **kws)

    plot a horizontal line spanning the plot axes

    Args:
        y (float):  y position of line
        xmin (float): starting x fraction (window units -- not user units!)
        xmax (float): ending x fraction (window units -- not user units!)
        win (int): index of plot window

    """
    plotter = get_plot_window(win=win, size=size)
    if plotter is None:
        return
    plotter.Raise()
    if 'label' not in kws:
        kws['label'] = '_nolegend_'
    plotter.panel.axes.axhline(y, xmin=xmin, xmax=xmax, **kws)
    if delay_draw:
        plotter.panel.canvas.draw()

def plot_axvline(x, ymin=0, ymax=1, win=1, size=None,
                 delay_draw=False, **kws):
    """plot_axvline(y, xmin=None, ymin=None, **kws)

    plot a vertical line spanning the plot axes

    Args:
        x (float):   x position of line
        ymin (float): starting y fraction (window units -- not user units!)
        ymax (float): ending y fraction (window units -- not user units!)
        win (int): index of plot window

    """
    plotter = get_plot_window(win=win, size=size)
    if plotter is None:
        return
    plotter.Raise()
    if 'label' not in kws:
        kws['label'] = '_nolegend_'
    plotter.panel.axes.axvline(x, ymin=ymin, ymax=ymax, **kws)
    if not delay_draw:
        plotter.panel.canvas.draw()


def hist(x, bins=10, win=1, new=False, size=None,
         force_draw=True, title=None, *args, **kws):
    """display a histogram

    Args:
        x (array-like): array of values
        bins (int): number of bins to show
        win (int): index of plot window

    """

    plotter = get_plot_window(win=win, size=size)
    if plotter is None:
        return
    plotter.Raise()
    if new:
        plotter.panel.axes.clear()
    out = plotter.panel.axes.hist(x, bins=bins, **kws)
    if title is not None:
        plotter.panel.set_title(title)
    plotter.panel.canvas.draw()

    return out

def imshow(map, y=None, x=None, colormap=None, win=1,
           wintitle=None, size=None, **kws):
    """imshow(map, ...)

    Display an 2-D array of intensities as a false-color map

    Args:
        map(ndarray):  map array data (see Note 1)
        y (array-like): values for pixels along vertical direction
        x (array-like): values for pixels along horizontal direction
        colormap (str): name of colormap to apply
        win (int): index of Image Window (1 to %d)
        size (tuple): width, height in pixels of Image Window
        wintitle(str): text for Window title [Image Window N]
        xlabel (str): label for horizontal axis ['X']
        ylabel (str): label for horizontal axis ['Y']
        style (str): display style ('image' or 'contour') ['image']
        nlevels (int): number of levels for contour
        contour_labels (bool): whether to show contour labels [True]
        show_axis (bool): whether to shos Axis [False]
        contrast_level (float): percent level for contrast [0]

    Returns:
        img, an ImageFrame

    Notes:
        1. the map data can either be a 2d array (shape NY, NX) for
           single-color map or (NY, NX, 3) array for an RGB map


    """
    img = get_image_window(win=win, size=size,
                          wintitle=wintitle)
    if img is not None:
        img.display(map, x=x, y=y, colormap=colormap, **kws)
    return img

def contour(map, x=None, y=None, **kws):
    """contour(map, ...)

    Display an 2-D array of intensities as a contour plot

    Notes:
        This is equivalent to
        imshow(map, ..., style='contour')
    """
    kws.update(dict(style='contour'))
    imshow(map, x=x, y=y, **kws)



#!/usr/bin/python
"""
wxmplot ImageFrame: a wx.Frame for image display, using matplotlib
"""
import os
from functools import partial

import wx
from wx.lib.agw.floatspin import FloatSpin, EVT_FLOATSPIN
import wx.lib.colourselect as csel

import numpy as np

import matplotlib.cm as cmap
from matplotlib.figure import Figure
from matplotlib.ticker import NullFormatter
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas

from wxutils import get_cwd

from .imagepanel import ImagePanel
from .imageconf import (ColorMap_List, Interp_List, Contrast_List,
                        Contrast_NDArray, Slices_List, RGB_COLORS,
                        ImageConfigFrame)
from .baseframe import BaseFrame
from .plotframe import PlotFrame
from .colors import rgb2hex, mpl_color
from .utils import LabeledTextCtrl, MenuItem, pack, gformat


CURSOR_MENULABELS = {'zoom':  ('Zoom to Rectangle\tCtrl+B',
                               'Left-Drag to zoom to rectangular box'),
                     'lasso': ('Select Points\tCtrl+V',
                               'Left-Drag to select points freehand'),
                     'prof':  ('Select Line Profile\tCtrl+K',
                               'Left-Drag to select like for profile')}

class ColorMapPanel(wx.Panel):
    """color map interface"""
    def __init__(self, parent, imgpanel, color=0,
                 colormap_list=None, default=None,
                 cmap_callback=None, title='Color Table',  **kws):
        wx.Panel.__init__(self, parent, -1,  **kws)

        self.imgpanel = imgpanel
        self.icol = color
        self.cmap_callback = cmap_callback

        labstyle = wx.ALIGN_LEFT|wx.LEFT|wx.TOP|wx.EXPAND
        sizer = wx.GridBagSizer(2, 2)

        self.title = wx.StaticText(self, label=title, size=(120, -1))
        sizer.Add(self.title, (0, 0), (1, 4), labstyle, 2)

        self.cmap_choice = None
        reverse = False
        cmapname = default
        if colormap_list is not None:
            cmap_choice =  wx.Choice(self, size=(90, -1), choices=colormap_list)
            cmap_choice.Bind(wx.EVT_CHOICE,  self.onCMap)
            self.cmap_choice = cmap_choice

            if cmapname is None:
                cmapname = colormap_list[0]

            if cmapname.endswith('_r'):
                reverse = True
                cmapname = cmap[:-2]
            cmap_choice.SetStringSelection(cmapname)

            cmap_reverse = wx.CheckBox(self, label='Reverse', size=(60, -1))
            cmap_reverse.Bind(wx.EVT_CHECKBOX, self.onCMapReverse)
            cmap_reverse.SetValue(reverse)
            self.cmap_reverse = cmap_reverse

        if cmapname is None:
            cmapname = 'gray'
        self.imgpanel.conf.cmap[color] = cmap.get_cmap(cmapname)

        maxval = self.imgpanel.conf.cmap_range
        wd, ht = 1.00, 0.125

        self.cmap_dat = np.outer(np.ones(int(maxval*ht)),
                                 np.linspace(0, 1, maxval))

        fig = Figure((wd, ht), dpi=150)

        ax = fig.add_axes([0, 0, 1, 1])
        ax.set_axis_off()
        self.cmap_canvas = FigureCanvas(self, -1, figure=fig)

        self.cmap_img = ax.imshow(self.cmap_dat, cmap=cmapname,
                                  interpolation='bilinear')
        self.cmap_lo = wx.Slider(self, -1, 0, 0, maxval,
                                 style=wx.SL_HORIZONTAL)

        self.cmap_hi = wx.Slider(self, -1, maxval, 0, maxval,
                                 style=wx.SL_HORIZONTAL)


        self.cmap_lo.Bind(wx.EVT_SCROLL,  self.onStretchLow)
        self.cmap_hi.Bind(wx.EVT_SCROLL,  self.onStretchHigh)

        irow = 0
        if self.cmap_choice is not None:
            irow += 1
            sizer.Add(self.cmap_choice,  (irow, 0), (1, 2), labstyle, 2)
            sizer.Add(self.cmap_reverse, (irow, 2), (1, 2), labstyle, 2)

        irow += 1
        sizer.Add(self.cmap_hi,      (irow, 0), (1, 4), labstyle, 2)
        irow += 1
        sizer.Add(self.cmap_canvas,  (irow, 0), (1, 4), wx.ALIGN_CENTER, 0)
        irow += 1
        sizer.Add(self.cmap_lo,      (irow, 0), (1, 4), labstyle, 2)

        self.imin_val = LabeledTextCtrl(self, 0, size=(80, -1),
                                        labeltext='Range:',
                                        action=partial(self.onThreshold, argu='lo'))
        self.imax_val = LabeledTextCtrl(self, maxval, size=(80, -1), labeltext=':',
                                        action=partial(self.onThreshold, argu='hi'))
        self.islider_range = wx.StaticText(self, label='Shown: ',
                                                size=(90, -1))
        irow += 1
        sizer.Add(self.imin_val.label, (irow, 0), (1, 1), labstyle, 1)
        sizer.Add(self.imin_val,       (irow, 1), (1, 1), labstyle, 0)
        sizer.Add(self.imax_val.label, (irow, 2), (1, 1), labstyle, 0)
        sizer.Add(self.imax_val,       (irow, 3), (1, 1), labstyle, 0)

        irow += 1
        sizer.Add(self.islider_range,  (irow, 0), (1, 4), labstyle, 0)

        pack(self, sizer)
        self.set_colormap(cmapname)


    def onCMap(self, event=None):
        if event is not None:
            self.set_colormap(name=event.GetString())
            self.imgpanel.redraw()
            if callable(self.cmap_callback):
                self.cmap_callback(name=event.GetString())


    def onCMapReverse(self, event=None):
        self.set_colormap()
        self.imgpanel.redraw()

    def set_colormap(self, name=None):
        conf = self.imgpanel.conf
        try:
            if name is None:
                name = self.cmap_choice.GetStringSelection()
        except:
            return

        try:
            reverse = (1 == int(self.cmap_reverse.GetValue()))
        except:
            reverse = False

        conf.set_colormap(name, reverse=reverse, icol=self.icol)
        self.redraw_cmap()


    def onThreshold(self, event=None, argu='hi'):
        col = self.icol
        conf = self.imgpanel.conf
        if (wx.EVT_TEXT_ENTER.evtType[0] == event.GetEventType()):
            try:
                val =  float(str(event.GetString()).strip())
            except:
                return
        elif (wx.EVT_KILL_FOCUS.evtType[0] == event.GetEventType()):
            val = float(self.imax_val.GetValue())
            if argu == 'lo':
                val = float(self.imin_val.GetValue())
        if argu == 'lo':
            conf.int_lo[col] = val
        else:
            conf.int_hi[col] = val
        lo = conf.int_lo[col]
        hi = conf.int_hi[col]
        self.islider_range.SetLabel('Shown: [ %.4g :  %.4g ]' % (lo, hi))
        self.imgpanel.redraw()

    def redraw_cmap(self):
        col = self.icol
        conf = self.imgpanel.conf
        self.cmap_img.set_cmap(conf.cmap[col])
        lo = int(conf.cmap_lo[col])
        hi = int(conf.cmap_hi[col])
        self.cmap_dat[:, :lo] = 0
        self.cmap_dat[:, lo:hi]  = np.linspace(0., 1., hi-lo)
        self.cmap_dat[:, hi:] = 1
        self.cmap_img.set_data(self.cmap_dat)
        self.cmap_canvas.draw()

    def onStretchLow(self, event=None):
        high = self.cmap_hi.GetValue()
        self.StretchCMap(event.GetInt(), high)

    def onStretchHigh(self, event=None):
        low = self.cmap_lo.GetValue()
        self.StretchCMap(low, event.GetInt())

    def StretchCMap(self, low, high):
        col = self.icol
        conf = self.imgpanel.conf
        lo, hi = min(low, high), max(low, high)
        if (hi-lo)<2:
            hi = min(hi, conf.cmap_range)
            lo = max(lo, 0)
        self.cmap_lo.SetValue(lo)
        self.cmap_hi.SetValue(hi)
        conf.cmap_lo[col] = lo
        conf.cmap_hi[col] = hi
        imin = float(self.imin_val.GetValue())
        imax = float(self.imax_val.GetValue())
        xlo = imin + (imax-imin)*lo/conf.cmap_range
        xhi = imin + (imax-imin)*hi/conf.cmap_range
        self.islider_range.SetLabel('Shown: [%.4g: %.4g]' % (xlo, xhi))
        self.redraw_cmap()
        self.imgpanel.redraw()


class InterpPanel(wx.Panel):
    """interpoloation / smoothing panel"""
    def __init__(self, parent, imgpanel, default=0, callback=None, **kws):
        wx.Panel.__init__(self, parent, -1,  **kws)

        self.imgpanel = imgpanel
        self.callback = callback
        labstyle = wx.ALIGN_LEFT|wx.LEFT|wx.TOP|wx.EXPAND
        sizer = wx.GridBagSizer(2, 2)

        title = wx.StaticText(self, label='Smoothing:', size=(120, -1))
        sizer.Add(title, (0, 0), (1, 1), labstyle, 2)

        interp_choice =  wx.Choice(self, size=(100, -1), choices=Interp_List)
        interp_choice.Bind(wx.EVT_CHOICE,  self.onInterp)
        interp_choice.SetSelection(default)

        sizer.Add(interp_choice,  (0, 1), (1, 2), labstyle, 2)
        pack(self, sizer)

    def onInterp(self, event=None):
        name=event.GetString()
        if name not in Interp_List:
            name = Interp_List[0]
        self.imgpanel.conf.interp = name
        self.imgpanel.redraw()
        if callable(self.callback):
            self.callback(name=name)

class ContrastPanel(wx.Panel):
    """auto-contrast panel"""
    def __init__(self, parent, default=0, callback=None, **kws):
        wx.Panel.__init__(self, parent, -1,  **kws)

        self.callback = callback
        labstyle = wx.ALIGN_LEFT|wx.LEFT|wx.TOP|wx.EXPAND
        sizer = wx.GridBagSizer(2, 2)

        title = wx.StaticText(self, label='Auto-Contrast (%):', size=(120, -1))
        sizer.Add(title, (0, 0), (1, 1), labstyle, 2)

        self.choice = wx.Choice(self, size=(100, -1), choices=Contrast_List)
        self.choice.Bind(wx.EVT_CHOICE,  self.onChoice)
        self.choice.SetSelection(default)
        sizer.Add(self.choice, (0, 1), (1, 1), labstyle, 2)
        pack(self, sizer)

    def set(self, choice=None):
        if choice in Contrast_List:
            self.SetStringSelection(choice)

    def advance(self):
        clevel = 1 + self.choice.GetSelection()
        if clevel >= len(Contrast_List):
            clevel = 0
        self.choice.SetSelection(clevel)
        clevel = Contrast_List[clevel]
        if clevel == 'None':
            clevel = 0
        else:
            clevel = float(clevel)
        self.callback(contrast_level=clevel)

    def retreat(self):
        clevel = -1 + self.choice.GetSelection()
        if clevel < 0:
            clevel = len(Contrast_List) - 1
        self.choice.SetSelection(clevel)
        clevel = Contrast_List[clevel]
        if clevel == 'None':
            clevel = 0
        else:
            clevel = float(clevel)
        self.callback(contrast_level=clevel)


    def onChoice(self, event=None):
        if callable(self.callback):
            clevel = event.GetString()
            if clevel == 'None':
                clevel = 0
            else:
                clevel = float(clevel)
            self.callback(contrast_level=clevel)




class ImageFrame(BaseFrame):
    """
    MatPlotlib Image Display ons a wx.Frame, using ImagePanel
    """

    help_msg =  """Quick help:

Left-Click:   to display X,Y coordinates and Intensity
Left-Drag:    to zoom in on region
Right-Click:  display popup menu with choices:
               Zoom out 1 level
               Zoom all the way out
               --------------------
               Rotate Image
               Save Image

Keyboard Shortcuts:   (For Mac OSX, replace 'Ctrl' with 'Apple')
  Saving Images:
     Ctrl-S:     Save image to file
     Ctrl-C:     Copy image to clipboard
     Ctrl-P:     Print Image

  Zooming:
     Ctrl-Z:     Zoom all the way out

  Rotating/Flipping:
     Ctrl-R:     Rotate Clockwise
     Ctrl-T:     Flip Top/Bottom
     Ctrl-F:     Flip Left/Right

"""

    def __init__(self, parent=None, size=(750, 625), lasso_callback=None,
                 mode='intensity', show_xsections=False,
                 cursor_labels=None, output_title='Image', subtitles=None,
                 user_menus=None, title='Image Display Frame', **kws):

        self.lasso_callback = lasso_callback
        self.user_menus = user_menus
        self.cursor_menulabels =  {}
        self.cursor_menulabels.update(CURSOR_MENULABELS)
        if cursor_labels is not None:
            self.cursor_menulabels.update(cursor_labels)

        BaseFrame.__init__(self, parent=parent, title=title,
                           output_title=output_title, size=size, **kws)

        self.cmap_panels = {}
        self.subtitles = {}
        self.config_mode = None
        if subtitles is not None:
            self.subtitles = subtitles
        sbar_widths = [-2, -1, -1]
        sbar = self.CreateStatusBar(len(sbar_widths), wx.CAPTION)
        sfont = sbar.GetFont()
        sfont.SetWeight(wx.BOLD)
        sfont.SetPointSize(10)
        sbar.SetFont(sfont)
        self.SetStatusWidths(sbar_widths)

        self.optional_menus = []
        self.win_config = None
        self.bgcol = rgb2hex(self.GetBackgroundColour()[:3])

        self.panel = ImagePanel(self, data_callback=self.onDataChange,
                                size=(650, 725), dpi=100,
                                lasso_callback=self.onLasso,
                                output_title=self.output_title)

        self.panel.nstatusbar = sbar.GetFieldsCount()
        self.BuildMenu()

        self.SetBackgroundColour('#F8F8F4')

        self.config_panel = wx.Panel(self)
        self.imin_val = {}
        self.imax_val = {}
        self.islider_range = {}

        self.config_mode = 'int'
        if mode.lower().startswith('rgb'):
            self.config_mode = 'rgb'

        self.Build_ConfigPanel()

        self.panel.messenger = self.write_message

        mainsizer = wx.BoxSizer(wx.HORIZONTAL)

        mainsizer.Add(self.config_panel, 0,
                      wx.LEFT|wx.ALIGN_LEFT|wx.TOP|wx.ALIGN_TOP|wx.EXPAND)

        mainsizer.Add(self.panel,  1, wx.ALL|wx.GROW)
        self.SetSizer(mainsizer)
        self.SetSize(self.GetBestVirtualSize())

    def display(self, img, title=None, colormap=None, style='image',
                subtitles=None, auto_contrast=False, contrast_level=None, **kws):
        """display image"""
        if title is not None:
            self.SetTitle(title)
        if subtitles is not None:
            self.subtitles = subtitles
        cmode = self.config_mode.lower()[:3]
        img = np.array(img)

        if len(img.shape) == 3:
            ishape = img.shape
            # make sure 3d image is shaped (NY, NX, 3)
            if ishape[2] != 3:
                if ishape[0] == 3:
                    img = img.swapaxes(0, 1).swapaxes(1, 2)
                elif ishape[1] == 3:
                    img = img.swapaxes(1, 2)

            if cmode != 'rgb':
                for comp in self.config_panel.Children:
                    comp.Destroy()
                self.config_mode = 'rgb'
                self.panel.conf.tricolor_mode = 'rgb'
                self.Build_ConfigPanel()
        else:
            if cmode != 'int':
                for comp in self.config_panel.Children:
                    comp.Destroy()
                self.config_mode = 'int'
                self.Build_ConfigPanel()

        if contrast_level is None:
            if auto_contrast:
                cl_str = '1.0'
            else:
                cl_str = self.contrast_panel.choice.GetStringSelection()
        else:
            cl_int = max(np.where(Contrast_NDArray<=contrast_level)[0])
            cl_str = Contrast_List[cl_int]

        if cl_str == 'None':
            contrast_level = 0
        else:
            contrast_level = float(cl_str)

        self.contrast_panel.choice.SetStringSelection(cl_str)
        self.panel.conf.contrast_level = contrast_level

        self.panel.display(img, style=style, contrast_level=contrast_level,
                           colormap=colormap, **kws)

        self.set_contrast_levels(contrast_level=contrast_level)

        self.panel.conf.title = title
        if colormap is not None and self.config_mode == 'int':
            self.cmap_panels[0].cmap_choice.SetStringSelection(colormap)
            self.cmap_panels[0].set_colormap(name=colormap)

        if subtitles is not None:
            if isinstance(subtitles, dict):
                self.set_subtitles(**subtitles)
            elif self.config_mode == 'int':
                self.set_subtitles(red=subtitles)

        self.panel.conf.style = 'image'
        self.contrast_panel.Enable()

        if style == 'contour':
            self.panel.conf.style = 'contour'
            self.contrast_panel.Disable()
            # self.interp_panel.Disable()

        self.config_panel.Refresh()
        self.SendSizeEvent()
        wx.CallAfter(self.EnableMenus)

    def configure(self, event=None):
        """show configuration frame"""
        if self.win_config is not None:
            try:
                self.win_config.Raise()
            except:
                self.win_config = None

        if self.win_config is None:
            self.win_config = ImageConfigFrame(parent=self,
                                               config=self.panel.conf)

            self.win_config.Raise()


    def set_subtitles(self, red=None, green=None, blue=None, **kws):
        if self.config_mode.startswith('int') and red is not None:
            self.cmap_panels[0].title.SetLabel(red)

        if self.config_mode.startswith('rgb') and red is not None:
            self.cmap_panels[0].title.SetLabel(red)

        if self.config_mode.startswith('rgb') and green is not None:
            self.cmap_panels[1].title.SetLabel(green)

        if self.config_mode.startswith('rgb') and blue is not None:
            self.cmap_panels[2].title.SetLabel(blue)


    def EnableMenus(self, evt=None):
        is_3color = len(self.panel.conf.data.shape) > 2
        for menu, on_3color in self.optional_menus:
            menu.Enable(is_3color==on_3color)

    def BuildMenu(self):
        # file menu
        mfile = self.Build_FileMenu(extras=(('Save Image of Colormap',
                                     'Save Image of Colormap',
                                      self.onCMapSave),))

        # image menua
        mview = self.view_menu = wx.Menu()
        conf = self.panel.conf


        MenuItem(self, mview, "Zoom Out\tCtrl+Z", "Zoom out to full data range",
                 self.panel.unzoom)

        MenuItem(self, mview, "Configure\tCtrl+K", "Configure Image Options",
                 self.configure)

        mview.AppendSeparator()
        MenuItem(self, mview, 'Enhance Contrast Cycle\tCtrl++',
                 'Cycle Through Contrast Choices', self.cycle_contrast)

        MenuItem(self, mview, 'Reduce Contrast Cycle\tCtrl+-',
                 'Cycle Through Contrast Choices',
                 partial(self.cycle_contrast, dir='back'))


        mview.AppendSeparator()
        MenuItem(self, mview, 'Show Histogram\tCtrl+H',
                 'Show Intensity Histogram', self.show_histogram)

        m = MenuItem(self, mview, 'Toggle Axes Labels\tCtrl+A',
                     'Toggle displacy of Axis labels',
                     self.onAxesLabels, kind=wx.ITEM_CHECK,
                     checked=conf.show_axis)

        m = MenuItem(self, mview, 'Toggle Grid Lines\tCtrl+G',
                     'Toggle displacy of Grid Lines at Axis labels',
                     self.onAxesGrid, kind=wx.ITEM_CHECK,
                     checked=conf.show_grid)

        m = MenuItem(self, mview, 'Toggle Contour Plot\tCtrl+N',
                     'Shown as Contour Plot',
                     self.onContourToggle, kind=wx.ITEM_CHECK,
                     checked=conf.style=='contour')
        self.optional_menus.append((m, False))

        m = MenuItem(self, mview, 'Toggle Scalebar\tCtrl+B', 'Show Scalebar',
                     self.onScalebarToggle, checked=conf.scalebar_show,
                     kind=wx.ITEM_CHECK)

        m = MenuItem(self, mview,
                     'Toggle Background Color (Black/White)\tCtrl+W',
                     'Toggle background color for 3-color images',
                     self.onTriColorBG, kind=wx.ITEM_CHECK,
                     checked=conf.tricolor_bg == 'white')
        self.optional_menus.append((m, True))

        mslice = wx.Menu()
        m1 = MenuItem(self, mslice, 'Show No X/Y Slices', 'Do not show X/Y slices',
                      self.onSliceChoice, kind=wx.ITEM_RADIO)
        m2 = MenuItem(self, mslice, 'Show X (Horizontal) Slices\tCtrl+x',
                      'show X slices',
                      self.onSliceChoice, kind=wx.ITEM_RADIO)
        m3 = MenuItem(self, mslice, 'Show Y (Vertical) Slices\tCtrl+Y', 'show Y slices',
                      self.onSliceChoice, kind=wx.ITEM_RADIO)
        self.slice_menus = {m1.GetId(): 'None', m2.GetId(): 'X', m3.GetId(): 'Y'}
        m = MenuItem(self, mslice, 'Slices Follow Mouse Motion?',
                     'Update Slices on Mouse Motion',
                     self.onSliceDynamic, checked=conf.slice_onmotion,
                     kind=wx.ITEM_CHECK)



        mrot = wx.Menu()
        MenuItem(self, mrot, 'Rotate clockwise\tCtrl+R', '',
                 partial(self.onFlip, mode='rot_cw'))
        MenuItem(self, mrot,  'Flip Top/Bottom\tCtrl+T', '',
                 partial(self.onFlip, mode='flip_ud'))
        MenuItem(self, mrot,  'Flip Left/Right\tCtrl+F', '',
                 partial(self.onFlip, mode='flip_lr'))
        MenuItem(self, mrot,  'Reset Flips/Rotations', '',
                 partial(self.onFlip, mode='restore'))


        msmooth = wx.Menu()
        self.smooth_menus = {}
        for sname in Interp_List:
            m = MenuItem(self, msmooth, sname, sname,
                         self.onSmoothChoice, kind=wx.ITEM_RADIO)
            self.smooth_menus[m.GetId()] = sname

        # help
        mhelp = wx.Menu()
        MenuItem(self, mhelp, 'Quick Reference',
                 'Quick Reference for WXMPlot', self.onHelp)
        MenuItem(self, mhelp, 'About', 'About WXMPlot', self.onAbout)

        # add all sub-menus, including user-added
        submenus = [('File', mfile),
                    ('Image', mview),
                    ('X/Y Slices', mslice),
                    ('Orientation', mrot),
                    ('Smoothing', msmooth)]
        if self.user_menus is not None:
            submenus.extend(self.user_menus)

        submenus.append(('&Help', mhelp))

        mbar = wx.MenuBar()
        for title, menu in submenus:
            mbar.Append(menu, title)

        self.SetMenuBar(mbar)
        self.Bind(wx.EVT_CLOSE,self.onExit)

    def onCursorMode(self, event=None, mode='zoom'):
        self.panel.cursor_mode = mode

    def onFlip(self, event=None, mode=None):
        panel = self.panel
        if mode == 'flip_lr':
            panel.flip_horiz()
        elif mode == 'flip_ud':
            panel.flip_vert()
        elif mode == 'rot_cw':
            panel.rotate90()
        elif mode == 'restore':
            panel.restore_flips_rotations()
        panel.unzoom_all()

    def Build_ConfigPanel(self):
        """config panel for left-hand-side of frame: RGB Maps"""
        panel = self.config_panel

        sizer = wx.BoxSizer(wx.VERTICAL)
        lsty = wx.ALIGN_LEFT|wx.LEFT|wx.TOP|wx.EXPAND

        self.contrast_panel = ContrastPanel(panel,
                                            callback=self.set_contrast_levels)

        if self.config_mode == 'rgb':
            for icol, col in enumerate(RGB_COLORS):
                self.cmap_panels[icol] =  ColorMapPanel(panel, self.panel,
                                                        title='%s: ' % col.title(),
                                                        color=icol,
                                                        default=col,
                                                        colormap_list=None)
                sizer.Add(self.cmap_panels[icol], 0, lsty, 2)

        else:
            self.cmap_panels[0] =  ColorMapPanel(panel, self.panel,
                                                 default='gray',
                                                 colormap_list=ColorMap_List)

            sizer.Add(self.cmap_panels[0],  0, lsty, 1)

        sizer.Add(self.contrast_panel, 0, lsty, 2)
        sizer.Add(wx.StaticLine(panel, size=(100, 2),
                                style=wx.LI_HORIZONTAL), 0, lsty, 2)

        cust = self.CustomConfig(panel, None, 0)
        if cust is not None:
            sizer.Add(cust, 0, lsty, 1)
        pack(panel, sizer)


    def CustomConfig(self, lpanel, lsizer, irow):
        """ override to add custom config panel items
        to bottom of config panel
        """
        pass

    def onSliceChoice(self, event=None):
        if event is not None:
            name = self.slice_menus.get(event.GetId(), Slices_List[0])
            self.panel.conf.slices = name
        self.panel.update_slices()

    def onSliceDynamic(self, event=None):
        conf  = self.panel.conf
        conf.slice_onmotion = not conf.slice_onmotion

    def onScalebarToggle(self, event=None):
        conf  = self.panel.conf
        conf.scalebar_show = not conf.scalebar_show
        self.panel.redraw()


    def onSmoothChoice(self, event=None):
        name = self.smooth_menus.get(event.GetId(), Interp_List[0])
        self.panel.conf.interp = name
        self.panel.redraw()

    def onContourToggle(self, event=None):
        if len(self.panel.conf.data.shape) > 2:
            return
        panel = self.panel
        conf  = panel.conf

        is_contour = event is None or event.IsChecked()
        conf.style = {True: 'contour', False: 'image'}[is_contour]
        self.contrast_panel.Enable(not is_contour)

        nlevels = int(conf.ncontour_levels)
        if self.config_mode == 'int':
            self.cmap_panels[0].set_colormap()
        panel.axes.cla()
        panel.display(conf.data, x=conf.xdata, y=conf.ydata, nlevels=nlevels,
                      contour_labels=conf.contour_labels, style=conf.style,
                      xlabel=conf.xlab, ylabel=conf.ylab)
        panel.redraw()

    def onAxesLabels(self, event=None):
        conf = self.panel.conf
        conf.show_axis = not conf.show_axis
        self.panel.autoset_margins()
        self.panel.redraw()

    def onAxesGrid(self, event=None):
        conf = self.panel.conf
        conf.show_grid = not conf.show_grid
        if conf.show_axis:
            self.panel.autoset_margins()
            self.panel.redraw()

    def onSliceMotion(self, event=None):
        conf = self.panel.conf
        conf.slice_onmotion = not conf.slice_onmotion
        self.panel.redraw()

    def onTriColorBG(self, event=None):
        bgcol = {True:'white', False:'black'}[event.IsChecked()]
        conf = self.panel.conf
        if bgcol == conf.tricolor_bg:
            return

        conf.tricolor_bg = bgcol
        cmaps = colors = RGB_COLORS
        if bgcol.startswith('wh'):
            cmaps = ('Reds', 'Greens', 'Blues')

        self.cmap_panels[0].set_colormap(name=cmaps[0])
        self.cmap_panels[1].set_colormap(name=cmaps[1])
        self.cmap_panels[2].set_colormap(name=cmaps[2])
        self.panel.redraw()

    def onLasso(self, data=None, selected=None, mask=None, **kws):
        if hasattr(self.lasso_callback , '__call__'):
            self.lasso_callback(data=data, selected=selected, mask=mask, **kws)

    def onDataChange(self, data, x=None, y=None, col='int', **kw):
        conf = self.panel.conf
        if len(data.shape) == 2: # intensity map
            imin, imax = data.min(), data.max()
            conf.int_lo[0] = imin
            conf.int_hi[0] = imax
            cpan = self.cmap_panels[0]

            cpan.cmap_lo.SetValue(imin)
            cpan.cmap_hi.SetValue(imax)

            cpan.imin_val.SetValue('%.4g' % imin)
            cpan.imax_val.SetValue('%.4g' % imax)
            cpan.imin_val.Enable()
            cpan.imax_val.Enable()

        else:
            for ix in range(3):
                imin, imax = data[:,:,ix].min(), data[:,:,ix].max()
                conf.int_lo[ix] = imin
                conf.int_hi[ix] = imax
                self.cmap_panels[ix].imin_val.SetValue('%.4g' % imin)
                self.cmap_panels[ix].imax_val.SetValue('%.4g' % imax)
                self.cmap_panels[ix].imin_val.Enable()
                self.cmap_panels[ix].imax_val.Enable()

    def cycle_contrast(self, event=None, dir='fore'):
        if dir.startswith('f'):
            self.contrast_panel.advance()
        else:
            self.contrast_panel.retreat()

    def set_contrast_levels(self, contrast_level=None):
        """enhance contrast levels, or use full data range
        according to value of self.panel.conf.contrast_level
        """
        if contrast_level is None:
            clevel = self.contrast_panel.choice.GetStringSelection()
            if clevel == 'None':
                contrast_level = 0
            else:
                contrast_level = float(clevel)

        conf = self.panel.conf
        img  = self.panel.conf.data
        if contrast_level is None:
            contrast_level = 0
        conf.contrast_level = contrast_level
        clevels = [contrast_level, 100.0-contrast_level]

        if len(img.shape) == 2: # intensity map
            col = 0
            jmin = imin = img.min()
            jmax = imax = img.max()
            self.cmap_panels[col].imin_val.SetValue('%.4g' % imin)
            self.cmap_panels[col].imax_val.SetValue('%.4g' % imax)

            jmin, jmax = np.percentile(img, clevels)
            if imax == imin:
                imax = imin + 0.5
            conf.cmap_lo[col] = xlo = (jmin-imin)*conf.cmap_range/(imax-imin)
            conf.cmap_hi[col] = xhi = (jmax-imin)*conf.cmap_range/(imax-imin)


            self.cmap_panels[col].cmap_hi.SetValue(xhi)
            self.cmap_panels[col].cmap_lo.SetValue(xlo)
            self.cmap_panels[col].islider_range.SetLabel('Shown: [ %.4g :  %.4g ]' % (jmin, jmax))
            self.cmap_panels[col].redraw_cmap()

        if len(img.shape) == 3: # rgb map
            for ix in range(3):
                jmin = imin = img[:,:,ix].min()
                jmax = imax = img[:,:,ix].max()
                self.cmap_panels[ix].imin_val.SetValue('%.4g' % imin)
                self.cmap_panels[ix].imax_val.SetValue('%.4g' % imax)

                jmin, jmax = np.percentile(img[:,:,ix], clevels)
                if imax == imin:
                    imax = imin + 0.5
                conf.cmap_lo[ix] = xlo = (jmin-imin)*conf.cmap_range/(imax-imin)
                conf.cmap_hi[ix] = xhi = (jmax-imin)*conf.cmap_range/(imax-imin)
                self.cmap_panels[ix].cmap_hi.SetValue(xhi)
                self.cmap_panels[ix].cmap_lo.SetValue(xlo)

                self.cmap_panels[ix].islider_range.SetLabel('Shown: [ %.4g :  %.4g ]' % (jmin, jmax))
                self.cmap_panels[ix].redraw_cmap()
        self.panel.redraw()

    def show_grid(self, event=None):
        conf = self.panel.conf


    def show_histogram(self, event=None):
        conf = self.panel.conf
        img  = conf.data

        title = '%s: Histogram' % (self.GetTitle())
        dat, color = None, None

        if len(img.shape) == 2:
            nbins = min(101, img.size)
            dat = img.flatten()

        elif len(img.shape) == 3:
            nbins = int(min(101, img.size/3))
            color = ('red', 'green', 'blue')
            dat = [img[:,:,i].flatten() for i in range(3)]
            dat = np.array(dat).transpose()

        if dat is not None:
            pf = PlotFrame(title=title, parent=self)
            if color is None:
                color = pf.panel.conf.traces[0].color
            pf.panel.axes.hist(dat, bins=nbins, rwidth=0.75, color=color,
                               stacked=(len(dat.shape)==2))
            pf.panel.conf.relabel(xlabel='Intensity', ylabel='Population')
            pf.Raise()
            pf.Show()


    def onCMapSave(self, event=None, col='int'):
        """save color table image"""
        file_choices = 'PNG (*.png)|*.png'
        ofile = 'Colormap.png'
        dlg = wx.FileDialog(self, message='Save Colormap as...',
                            defaultDir=get_cwd(),
                            defaultFile=ofile,
                            wildcard=file_choices,
                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            self.cmap_panels[0].cmap_canvas.print_figure(dlg.GetPath(), dpi=600)

    def save_figure(self,event=None, transparent=True, dpi=600):
        """ save figure image to file"""
        if self.panel is not None:
            self.panel.save_figure(event=event,
                                   transparent=transparent, dpi=dpi)


    def ExportTextFile(self, fname, title='unknown map'):
        buff = ["# Map Data for %s" % title,
                "#-------------------------------------"]
        data = self.panel.conf.data
        narr = 1
        labels = [' Y', ' X']
        if len(data.shape) == 3:
            ny, nx, narr = data.shape
            labels.extend(['Map%d' % (i+1) for i in range(narr)])
        else:
            ny, nx = data.shape
            labels.append('Map')

        labels = [(' '*(11-len(l)) + l + ' ') for l in labels]
        buff.append("#%s" % ('  '.join(labels)))
        xdata = np.arange(nx)
        ydata = np.arange(ny)
        if self.panel.conf.xdata is not None:
            xdata = self.panel.conf.xdata
        if self.panel.conf.ydata is not None:
            ydata = self.panel.conf.ydata

        for iy in range(ny):
            for ix in range(nx):
                darr = [ydata[iy], xdata[ix]]
                if narr == 1:
                    darr.append(data[iy, ix])
                else:
                    darr.extend(data[iy, ix])
                buff.append("  ".join([gformat(arr, 12) for arr in darr]))
        buff.append("")
        with open(fname, 'w') as fout:
            fout.write("\n".join(buff))
        fout.close()



#!/usr/bin/python
##
## Image Matrix Frame:  2 correlated images in a 2x2 grid:
#
#     +-------------+--------------+
#     |             |              |
#     |   mage1     |  image 1+2   |
#     |             |              |
#     +-------------+--------------+
#     |             |              |
#     | correlation |   image2     |
#     | plot        |              |
#     +-------------+--------------+
##

import os
from functools import partial

import wx
from wxutils import get_cwd
import numpy as np
import matplotlib


HAS_IMAGE = False
try:
    from PIL import Image
    HAS_IMAGE = True
except ImportError:
    pass

from .baseframe  import BaseFrame
from .plotpanel  import PlotPanel
from .imagepanel import ImagePanel
from .imageframe import ColorMapPanel, InterpPanel, ContrastPanel
from .imageconf import ColorMap_List, Interp_List
from .colors import rgb2hex
from .utils import MenuItem, pack, fix_filename, gformat

COLORMAPS = ('blue', 'red', 'green', 'yellow', 'cyan', 'magenta')
CM_DEFS = ('blue', 'yellow')

###, 'Reds', 'Greens', 'Blues')

def color_complements(color):
    colors = list(COLORMAPS)
    colors.remove(color)
    return colors

def image2wxbitmap(img):
    "PIL image 2 wx bitmap"
    wximg = wx.Image(*img.size)
    wximg.SetData(img.tobytes())
    return wximg.ConvertToBitmap()

class ImageMatrixFrame(BaseFrame):
    """
    wx.Frame, with 3 ImagePanels and correlation plot for 2 map arrays
    """
    def __init__(self, parent=None, size=(900,600),
                 cursor_callback=None, title='Image Matrix', **kws):

        self.sel_mask = None
        self.xdata = None
        self.ydata = None
        self.cursor_callback = cursor_callback
        BaseFrame.__init__(self, parent=parent,
                           title=title, size=size, **kws)

        self.title = title
        self.cmap_panels= {}
        sbar = self.CreateStatusBar(2, wx.CAPTION)
        sfont = sbar.GetFont()
        sfont.SetWeight(wx.BOLD)
        sfont.SetPointSize(10)
        sbar.SetFont(sfont)

        self.SetStatusWidths([-2, -1])
        self.SetStatusText('', 0)

        self.bgcol = rgb2hex(self.GetBackgroundColour()[:3])
        splitter  = wx.SplitterWindow(self, style=wx.SP_LIVE_UPDATE)
        splitter.SetMinimumPaneSize(225)

        conf_panel = wx.Panel(splitter)
        main_panel = wx.Panel(splitter)

        self.config_panel = conf_panel

        # main panel
        self.plot_panel = PlotPanel(main_panel, size=(400, 325))
        self.plot_panel.axesmargins = (15, 15, 15, 15)
        self.plot_panel.conf.set_axes_style(style='open')
        self.plot_panel.cursor_mode = 'lasso'
        self.plot_panel.plot_type = 'scatter'
        self.plot_panel.lass_callback = self.on_corplot_lasso

        img_opts = dict(size=(400, 300),
                        redraw_callback=self.on_imageredraw,
                        zoom_callback=self.on_imagezoom)
        self.img1_panel = ImagePanel(main_panel, **img_opts)
        self.img2_panel = ImagePanel(main_panel, **img_opts)
        self.dual_panel = ImagePanel(main_panel, **img_opts)

        self.imgpanels = [self.img1_panel,
                          self.img2_panel,
                          self.dual_panel]


        lsty = wx.LEFT|wx.TOP|wx.EXPAND

        ir = 0
        self.wids = {}
        self.cmap_panels = [None, None]
        csizer = wx.BoxSizer(wx.VERTICAL)
        for i, imgpanel in enumerate((self.img1_panel, self.img2_panel)):
            self.cmap_panels[i] =  ColorMapPanel(conf_panel, imgpanel,
                                                 title='Map %i: ' % (i+1),
                                                 color=0,
                                                 default=CM_DEFS[i],
                                                 colormap_list=COLORMAPS,
                                                 cmap_callback=partial(self.onColorMap, index=i))

            csizer.Add(self.cmap_panels[i], 0, lsty, 2)
            csizer.Add(wx.StaticLine(conf_panel, size=(200, 2),
                                    style=wx.LI_HORIZONTAL), 0, lsty, 2)


        self.interp_panel = InterpPanel(self.config_panel, self.img1_panel, default=0,
                                        callback=self.onInterp)
        self.contrast_panel = ContrastPanel(self.config_panel,
                                            callback=self.set_contrast_levels)

        csizer.Add(self.interp_panel, 0, lsty, 2)
        csizer.Add(self.contrast_panel, 0, lsty, 2)

        cust = self.CustomConfig(conf_panel)
        if cust is not None:
            sizer.Add(cust, 0, lsty, 1)
        pack(conf_panel, csizer)

        for name, panel in (('corplot', self.plot_panel),
                            ('map1',    self.img1_panel),
                            ('map2',    self.img2_panel),
                            ('dual',    self.dual_panel)):

            panel.report_leftdown = partial(self.report_leftdown, name=name)
            panel.messenger = self.write_message

        sizer = wx.GridSizer(2, 2, 2, 2)
        lsty |= wx.GROW|wx.ALL|wx.EXPAND
        sizer.Add(self.img1_panel, 1, lsty, 2)
        sizer.Add(self.dual_panel, 1, lsty, 2)
        sizer.Add(self.plot_panel, 1, lsty, 2)
        sizer.Add(self.img2_panel, 1, lsty, 2)

        pack(main_panel, sizer)
        splitter.SplitVertically(conf_panel, main_panel, 1)

        self.BuildMenu()

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(splitter, 1, wx.GROW|wx.ALL, 5)
        pack(self, sizer)

        self.dualimage_needs_update = False
        self.dualimage_timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.onDualImageTimer, self.dualimage_timer)
        self.dualimage_timer.Start(150)


    def CustomConfig(self, parent):
        """
        override to add custom config panel items to bottom of config panel
        """
        pass

    def onColorMap(self, name=None, index=None):
        colors = color_complements(name)
        opanel = self.cmap_panels[0]
        if index == 0:
            opanel = self.cmap_panels[1]

        c1 = opanel.cmap_choice.GetStringSelection()
        opanel.cmap_choice.Clear()
        for c in colors:
            opanel.cmap_choice.Append(c)


        ic1 = (COLORMAPS.index(name) + 3) % 6
        if COLORMAPS[ic1] in colors:
            c1 = COLORMAPS[ic1]
        if c1  not in colors:
            c1 = colors[0]
        opanel.cmap_choice.SetStringSelection(c1)
        opanel.set_colormap(name=c1)
        opanel.imgpanel.redraw()

    def unzoom_all(self, event=None):
        self.unzoom(event=event)

    def unzoom(self, event=None):
        self.xzoom = slice(0, self.map1.shape[1]+1)
        self.yzoom = slice(0, self.map1.shape[0]+1)

        self.update_scatterplot(self.map1, self.map2)
        self.plot_panel.unzoom_all()
        for p in self.imgpanels:
            p.unzoom_all()

    def BuildMenu(self):
        # file menu
        mfile = self.Build_FileMenu()

        mview =  wx.Menu()

        MenuItem(self, mview, "Zoom Out\tCtrl+Z",
                 "Zoom out to full data range",
                 self.unzoom)

        MenuItem(self, mview, 'Enhance Contrast Cycle\tCtrl++',
                 'Cycle Through Contrast Choices',
                 self.cycle_contrast)
        MenuItem(self, mview, 'Reduce Contrast Cycle\tCtrl+-',
                 'Cycle Through Contrast Choices',
                 partial(self.cycle_contrast, dir='back'))

        mbar = wx.MenuBar()
        mbar.Append(mfile, 'File')
        mbar.Append(mview, 'Image')

        self.SetMenuBar(mbar)
        self.Bind(wx.EVT_CLOSE,self.onExit)


    def save_figure(self, event=None):
        if not HAS_IMAGE:
            return

        file_choices = "PNG (*.png)|*.png|SVG (*.svg)|*.svg|PDF (*.pdf)|*.pdf"
        ofile = "%s.png" % self.title
        dlg = wx.FileDialog(self, message='Save Figure as...',
                            defaultDir=get_cwd(),
                            defaultFile=ofile,
                            wildcard=file_choices,
                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)

        if dlg.ShowModal() != wx.ID_OK:
            return

        path = dlg.GetPath()
        img = self.make_composite_image()
        img.save(path)
        self.write_message('Saved plot to %s' % path)

    def make_composite_image(self):
        h, w = 0, 0
        def GetImage(panel):
            wximg = panel.canvas.bitmap.ConvertToImage()
            w, h = wximg.GetWidth(), wximg.GetHeight()
            img = Image.new( 'RGB', (w, h))
            img.frombytes(bytes(wximg.GetData()))
            return img, w, h

        img1, w1, h1 = GetImage(self.img1_panel)
        img2, w2, h2 = GetImage(self.img2_panel)
        dual, w3, h3 = GetImage(self.dual_panel)
        plot, w4, h4 = GetImage(self.plot_panel)

        w = (w1 + w2 + w3 + w4 ) / 4
        h = (h1 + h2 + h3 + h4 ) / 4

        img = Image.new('RGB', (2*w+2, 2*h+2))
        img.paste(img1, (0,   0))
        img.paste(img2, (1+w, 1+h))
        img.paste(dual, (1+w, 0))
        img.paste(plot, (0,   1+h))
        return img

    def Copy_to_Clipboard(self, event=None):
        img = self.make_composite_image()
        bmp_obj = wx.BitmapDataObject()
        bmp_obj.SetBitmap(image2wxbitmap(img))

        if not wx.TheClipboard.IsOpened():
            open_success = wx.TheClipboard.Open()
            if open_success:
                wx.TheClipboard.SetData(bmp_obj)
                wx.TheClipboard.Close()
                wx.TheClipboard.Flush()

    def PrintSetup(self, event=None):
        dlg = wx.MessageDialog(self, "Printing not Available",
                               "Save Image or Copy to Clipboard",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def PrintPreview(self, event=None):
        dlg = wx.MessageDialog(self, "Printing not Available",
                               "Save Image or Copy to Clipboard",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def Print(self, event=None):
        dlg = wx.MessageDialog(self, "Printing not Available",
                               "Save Image or Copy to Clipboard",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def onInterp(self, event=None, name=None):
        if name not in Interp_List:
            name = Interp_List[0]
        for ipanel in self.imgpanels:
            ipanel.conf.interp = name
            ipanel.redraw()

    def onExit(self, event=None):
        self.dualimage_timer.Stop()
        self.Destroy()

    def report_leftdown(self,event=None, name=''):
        if event is None:
            return
        if event.xdata is None or event.ydata is None:
            return
        ix, iy = int(round(event.xdata)), int(round(event.ydata))
        if (ix >= 0 and ix < self.map1.shape[1] and
            iy >= 0 and iy < self.map1.shape[0]):
            pos = ''
            if self.xdata is not None:
                pos = ' %s=%.4g,' % (self.xlabel, self.xdata[ix])
            if self.ydata is not None:
                pos = '%s %s=%.4g,' % (pos, self.ylabel, self.ydata[iy])

            d1, d2 = (self.map1[iy, ix], self.map2[iy, ix])
            msg = "Pixel [%i, %i],%s %s=%.4g, %s=%.4g" % (ix, iy, pos,
                                                          self.name1, d1,
                                                          self.name2, d2)
            self.write_message(msg, panel=0)

            if callable(self.cursor_callback):
                self.cursor_callback(x=event.xdata, y=event.ydata)

    def on_corplot_lasso(self, data=None, selected=None, mask=None):
        self.sel_mask = mask
        try:
            mask.shape = self.zoom_map1.shape
            self.sel_mask = 0 * self.map1
            self.sel_mask[self.yzoom, self.xzoom] = mask
            self.dualimage_needs_update = True
        except:
            pass

    def on_imagezoom(self, event=None, wid=0, limits=None):
        if wid in [w.GetId() for w in self.imgpanels]:
            lims = [int(x) for x in limits]
            for ipanel in self.imgpanels:
                ax = ipanel.fig.axes[0]
                axlims = {ax: lims}
                ipanel.conf.zoom_lims.append(axlims)
                ipanel.set_viewlimits()

        m1 = self.map1[lims[2]:lims[3], lims[0]:lims[1]]
        m2 = self.map2[lims[2]:lims[3], lims[0]:lims[1]]
        self.xzoom = slice(lims[0], lims[1])
        self.yzoom = slice(lims[2], lims[3])
        self.update_scatterplot(m1, m2)

    def on_imageredraw(self, event=None, wid=0):
        if wid in (self.img1_panel.GetId(), self.img2_panel.GetId()):
            self.dualimage_needs_update = True

    def display(self, map1, map2, title=None, name1='Map1', name2='Map2',
                xlabel='x', ylabel='y', x=None, y=None):
        self.map1 = map1
        self.map2 = map2
        self.name1 = name1
        self.name2 = name2
        self.xdata = x
        self.ydata = y
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.xzoom = slice(0, map1.shape[1]+1)
        self.yzoom = slice(0, map1.shape[0]+1)

        self.img1_panel.display(map1, x=x, y=y)
        self.img2_panel.display(map2, x=x, y=y)

        self.cmap_panels[0].title.SetLabel(name1)
        self.cmap_panels[1].title.SetLabel(name2)
        self.update_scatterplot(map1, map2)

        self.set_contrast_levels()
        self.dualimage_needs_update = True
        self.panel = self.img1_panel

    def update_scatterplot(self, x, y):
        self.zoom_map1 = x
        self.zoom_map2 = y
        _x = x.flatten()
        _y = y.flatten()
        xmax = max(_x) * 1.2
        xmin = min(_x) * 0.8
        ymax = max(_y) * 1.2
        ymin = min(_y) * 0.8
        self.plot_panel.clear()
        self.plot_panel.conf.labelfont.set_size(7)
        self.plot_panel.scatterplot(_x, _y, size=3,
                                    min=xmin, xmax=xmax,
                                    ymin=ymin, ymax=ymax,
                                    xlabel=self.name1, ylabel=self.name2,
                                    callback=self.on_corplot_lasso)

    def cycle_contrast(self, event=None, dir='fore'):
        if dir.startswith('f'):
            self.contrast_panel.advance()
        else:
            self.contrast_panel.retreat()

    def set_contrast_levels(self, contrast_level=0):
        """enhance contrast levels, or use full data range
        according to value of self.panel.conf.contrast_level
        """
        for cmap_panel, img_panel in zip((self.cmap_panels[0], self.cmap_panels[1]),
                                         (self.img1_panel, self.img2_panel)):
            conf = img_panel.conf
            img  = img_panel.conf.data
            if contrast_level is None:
                contrast_level = 0
            conf.contrast_level = contrast_level
            clevels = [contrast_level, 100.0-contrast_level]

            jmin = imin = img.min()
            jmax = imax = img.max()
            cmap_panel.imin_val.SetValue('%.4g' % imin)
            cmap_panel.imax_val.SetValue('%.4g' % imax)
            jmin, jmax = np.percentile(img, clevels)

            conf.int_lo[0]  = imin
            conf.int_hi[0]  = imax
            conf.cmap_lo[0] = xlo = (jmin-imin)*conf.cmap_range/(imax-imin)
            conf.cmap_hi[0] = xhi = (jmax-imin)*conf.cmap_range/(imax-imin)

            cmap_panel.cmap_hi.SetValue(xhi)
            cmap_panel.cmap_lo.SetValue(xlo)
            cmap_panel.islider_range.SetLabel('Shown: [ %.4g :  %.4g ]' % (jmin, jmax))
            cmap_panel.redraw_cmap()
            img_panel.redraw()

    def onDualImageTimer(self, event=None):
        if not self.dualimage_needs_update:
            return

        self.dualimage_needs_update = False

        ctabs = [] # [col1, col2]
        dimgs = [] # [dat1, dat2]

        for panel, dmap in zip((self.img1_panel, self.img2_panel),
                               (self.map1, self.map2)):
            try:
                col = panel.conf.cmap[0].name
                dat = dmap
                conf = panel.conf
                ilo = float(conf.int_lo[0])
                ihi = float(conf.int_hi[0])
                mlo = float(conf.cmap_lo[0])/(1.0*conf.cmap_range)
                mhi = float(conf.cmap_hi[0])/(1.0*conf.cmap_range)

                dat = (dat - ilo) / (ihi - ilo)
                dat = (dat - mlo) / (mhi - mlo)
                dat[np.where(dat<0)] = 0.0
                dat[np.where(dat>1)] = 1.0
                ctabs.append(col)
                dimgs.append(dat)
            except:
                return

        mr, mg, mb =(1, 1, 1)
        for col in ctabs:
            if col.startswith('red'):
                mr = 0
            elif col.startswith('green'):
                mg = 0
            elif col.startswith('blue'):
                mb = 0
            elif col.startswith('cyan'):
                mb = mg = 0
            elif col.startswith('yellow'):
                mr = mg = 0
            elif col.startswith('magenta'):
                mr = mb = 0

        maskcolor = {(1, 0, 0): 'red',
                     (0, 1, 0): 'green',
                     (0, 0, 1): 'blue',
                     (0, 0, 0): 'white',
                     (0, 1, 1): 'cyan',
                     (1, 1, 0): 'yellow',
                     (0, 1, 1): 'cyan',
                     (1, 0, 1): 'magenta'}.get((mr, mg, mb), 'white')

        if self.sel_mask is not None:
            ctabs.append(maskcolor)
            dimgs.append(self.sel_mask)

        w, h = dimgs[0].shape
        img = np.zeros(3*w*h).reshape(w, h, 3)

        for col, dimg in zip(ctabs, dimgs):
            if col.endswith('_r'):
                col = col[:-2]
                dimg = 1.0 - dimg
            if col == 'red':
                img[:, :, 0] += dimg
            elif col == 'green':
                img[:, :, 1] += dimg
            elif col == 'blue':
                img[:, :, 2] += dimg
            elif col == 'cyan':
                img[:, :, 1] += dimg/2.
                img[:, :, 2] += dimg/2.
            elif col == 'yellow':
                img[:, :, 0] += dimg/2.
                img[:, :, 1] += dimg/2.
            elif col == 'magenta':
                img[:, :, 0] += dimg/2.
                img[:, :, 2] += dimg/2.
            elif col == 'white':
                img[:, :, 0] += dimg
                img[:, :, 1] += dimg
                img[:, :, 2] += dimg

        dp = self.dual_panel
        try:
            dp.conf.image.set_data(img)
        except:
            dp.conf.data = img
        im1_zoom = self.img1_panel.conf.zoom_lims[-1]
        if im1_zoom is not None:
            dp.conf.zoom_lims.append({dp.axes:list(im1_zoom.values())[0]})
            dp.set_viewlimits()
            if self.sel_mask is not None:
                dp.conf.image = dp.axes.imshow(img, cmap=dp.conf.cmap[0],
                                               interpolation=dp.conf.interp)
            dp.canvas.draw()
        else:
            self.dual_panel.display(img)
        self.sel_mask = None


    def ExportTextFile(self, fname, title='unknown map'):
        buff = ["# Correlation Map Data for %s" % title,
                "#-------------------------------------"]

        labels = ['  Y', '  X', self.name1, self.name2]
        labels = [(' '*(11-len(l)) + l + ' ') for l in labels]

        buff.append("#%s" % ('  '.join(labels)))

        ny, nx = self.map1.shape
        xdat = np.arange(nx)
        ydat = np.arange(ny)
        if self.xdata is not None:
            xdat = self.xdata
        if self.ydata is not None:
            ydat = self.ydata

        for iy in range(ny):
            for ix in range(nx):
                d = [ydat[iy], xdat[ix], self.map1[iy, ix], self.map2[iy, ix]]
                buff.append("  ".join([gformat(a, 12) for a in d]))

        fout = open(fname, 'w')
        fout.write("%s\n" % "\n".join(buff))
        fout.close()



#!/usr/bin/python
##
## StackedPlotFrame: a wx.Frame for 2 PlotPanels, top and bottom
##   with the top panel being the main panel and the lower panel
##   being 1/4 the height (configurable) and the dependent panel

import wx
import numpy as np
import matplotlib
from matplotlib.ticker import NullFormatter, NullLocator
from functools import partial
from .utils import pack, MenuItem
from .plotpanel import PlotPanel
from .baseframe import BaseFrame

class StackedPlotFrame(BaseFrame):
    """
    Top/Bottom MatPlotlib panels in a single frame
    """
    def __init__(self, parent=None, title ='Stacked Plot Frame',
                 framesize=(550,650), panelsize=(550,350),
                 ratio=3.0, **kws):

        BaseFrame.__init__(self, parent=parent, title=title,
                           size=framesize, **kws)
        self.ratio = ratio
        self.panelsize = panelsize
        self.panel = None
        self.panel_bot = None
        self.xlabel = None
        self.BuildFrame()

    def get_panel(self, panelname):
        if panelname.lower().startswith('bot'):
            return self.panel_bot
        return self.panel

    def plot(self, x, y, panel='top', xlabel=None, **kws):
        """plot after clearing current plot """
        panel = self.get_panel(panel)
        panel.plot(x, y, **kws)
        if xlabel is not None:
            self.xlabel = xlabel
        if self.xlabel is not None:
            self.panel_bot.set_xlabel(self.xlabel)

    def oplot(self, x, y, panel='top', xlabel=None, **kws):
        """plot method, overplotting any existing plot """
        panel = self.get_panel(panel)
        panel.oplot(x, y, **kws)
        if xlabel is not None:
            self.xlabel = xlabel
        if self.xlabel is not None:
            self.panel_bot.set_xlabel(self.xlabel)

    def unzoom_all(self, event=None):
        """ zoom out full data range """
        for p in (self.panel, self.panel_bot):
            p.conf.zoom_lims = []
            p.conf.unzoom(full=True)
        # self.panel.set_viewlimits()

    def unzoom(self, event=None, panel='top'):
        """zoom out 1 level, or to full data range """
        panel = self.get_panel(panel)
        panel.conf.unzoom(event=event)
        self.panel.set_viewlimits()

    def update_line(self, t, x, y, panel='top', **kws):
        """overwrite data for trace t """
        panel = self.get_panel(panel)
        panel.update_line(t, x, y, **kws)

    def set_xylims(self, lims, axes=None, panel='top', **kws):
        """set xy limits"""
        panel = self.get_panel(panel)
        # print("Stacked set_xylims ", panel, self.panel)
        panel.set_xylims(lims, axes=axes, **kws)

    def clear(self, panel='top'):
        """clear plot """
        panel = self.get_panel(panel)
        panel.clear()

    def set_title(self,s, panel='top'):
        "set plot title"
        panel = self.get_panel(panel)
        panel.set_title(s)

    def set_xlabel(self,s, panel='top'):
        "set plot xlabel"
        self.panel_bot.set_xlabel(s)

    def set_ylabel(self,s, panel='top'):
        "set plot xlabel"
        panel = self.get_panel(panel)
        panel.set_ylabel(s)

    def save_figure(self, event=None, panel='top'):
        """ save figure image to file"""
        panel = self.get_panel(panel)
        panel.save_figure(event=event)

    def configure(self, event=None, panel='top'):
        panel = self.get_panel(panel)
        panel.configure(event=event)

    ####
    ## create GUI
    ####
    def BuildFrame(self):
        sbar = self.CreateStatusBar(2, wx.CAPTION)
        sfont = sbar.GetFont()
        sfont.SetWeight(wx.BOLD)
        sfont.SetPointSize(10)
        sbar.SetFont(sfont)

        self.SetStatusWidths([-3,-1])
        self.SetStatusText('',0)

        sizer = wx.BoxSizer(wx.VERTICAL)

        botsize = self.panelsize[0], self.panelsize[1]/self.ratio
        margins = {'top': dict(left=0.15, bottom=0.01, top=0.10, right=0.05),
                   'bot': dict(left=0.15, bottom=0.300, top=0.02, right=0.05)}

        self.panel     = PlotPanel(self, size=self.panelsize)
        self.panel_bot = PlotPanel(self, size=botsize)
        self.panel.xformatter = self.null_formatter
        lsize = self.panel.conf.labelfont.get_size()
        self.panel_bot.conf.labelfont.set_size(lsize-2)
        self.panel_bot.yformatter = self.bot_yformatter

        self.panel.conf.theme_color_callback = self.onThemeColor
        self.panel.conf.margin_callback = self.onMargins

        for pan, pname in ((self.panel, 'top'), (self.panel_bot, 'bot')):
            pan.messenger = self.write_message
            pan.conf.auto_margins = False
            pan.conf.set_margins(**margins[pname])
            # pan.axes.update_params()
            # pan.axes.set_position(pan.axes.figbox)
            # ax.update_params()
            # ax.set_position(ax.figbox)
            figpos = pan.axes.get_subplotspec().get_position(pan.canvas.figure)
            pan.axes.set_position(figpos)

            pan.set_viewlimits = partial(self.set_viewlimits, panel=pname)
            pan.unzoom_all = self.unzoom_all
            pan.unzoom = self.unzoom
            pan.canvas.figure.set_facecolor('#F4F4EC')

        # suppress mouse events on the bottom panel
        null_events = {'leftdown': None, 'leftup': None, 'rightdown': None,
                       'rightup': None, 'motion': None, 'keyevent': None}
        self.panel_bot.cursor_modes = {'zoom': null_events}


        sizer.Add(self.panel,     round(0.75*self.ratio), wx.GROW|wx.EXPAND, 2)
        sizer.Add(self.panel_bot, 1, wx.GROW, 2)
        pack(self, sizer)
        self.SetSize(self.GetBestVirtualSize())
        self.BuildMenu()

    def BuildMenu(self):
        mfile = self.Build_FileMenu()

        mopts = wx.Menu()
        MenuItem(self, mopts, "Configure Plot\tCtrl+K",
                 "Configure Plot styles, colors, labels, etc",
                 self.panel.configure)
        MenuItem(self, mopts, "Configure Lower Plot",
                 "Configure Plot styles, colors, labels, etc",
                 self.panel_bot.configure)
        MenuItem(self, mopts, "Toggle Legend\tCtrl+L",
                 "Toggle Legend Display",
                 self.panel.toggle_legend)
        MenuItem(self, mopts, "Toggle Grid\tCtrl+G",
                 "Toggle Grid Display",
                 self.toggle_grid)


        mopts.AppendSeparator()

        MenuItem(self, mopts, "Zoom Out\tCtrl+Z",
                 "Zoom out to full data range",
                 self.unzoom_all)

        mhelp = wx.Menu()
        MenuItem(self, mhelp, "Quick Reference",  "Quick Reference for WXMPlot", self.onHelp)
        MenuItem(self, mhelp, "About", "About WXMPlot", self.onAbout)

        mbar = wx.MenuBar()
        mbar.Append(mfile, 'File')
        mbar.Append(mopts, '&Options')
        if self.user_menus is not None:
            for title, menu in self.user_menus:
                mbar.Append(menu, title)

        mbar.Append(mhelp, '&Help')

        self.SetMenuBar(mbar)
        self.Bind(wx.EVT_CLOSE,self.onExit)

    def toggle_grid(self, event=None, show=None):
        "toggle grid on top/bottom panels"
        if show is None:
            show = not self.panel.conf.show_grid
        for p in (self.panel, self.panel_bot):
            p.conf.enable_grid(show)

    def onThemeColor(self, color, item):
        """pass theme colors to bottom panel"""
        bconf = self.panel_bot.conf
        if item == 'grid':
            bconf.set_gridcolor(color)
        elif item == 'bg':
            bconf.set_facecolor(color)
        elif item == 'frame':
            bconf.set_framecolor(color)
        elif item == 'text':
            bconf.set_textcolor(color)
        bconf.canvas.draw()

    def onMargins(self, left=0.1, top=0.1, right=0.1, bottom=0.1):
        """ pass left/right margins on to bottom panel"""
        bconf = self.panel_bot.conf
        l, t, r, b = bconf.margins
        bconf.set_margins(left=left, top=t, right=right, bottom=b)
        bconf.canvas.draw()

    def set_viewlimits(self, panel='top'):
        """update xy limits of a plot, as used with .update_line() """
        this_panel = self.get_panel(panel)

        xmin, xmax, ymin, ymax = this_panel.conf.set_viewlimits()[0]
        # print("Set ViewLimits ", xmin, xmax, ymin, ymax)
        # make top/bottom panel follow xlimits
        if this_panel == self.panel:
            other = self.panel_bot
            for _ax in other.fig.get_axes():
                _ax.set_xlim((xmin, xmax), emit=True)
            other.draw()

    def null_formatter(self, x, pos, type='x'):
        return ''

    def bot_yformatter(self, val, type=''):
        """custom formatter for FuncFormatter() and bottom panel"""
        fmt = '%1.5g'

        ax = self.panel_bot.axes.yaxis

        ticks = ax.get_major_locator()()
        dtick = ticks[1] - ticks[0]

        if   dtick > 29999:
            fmt = '%1.5g'
        elif dtick > 1.99:
            fmt = '%1.0f'
        elif dtick > 0.099:
            fmt = '%1.1f'
        elif dtick > 0.0099:
            fmt = '%1.2f'
        elif dtick > 0.00099:
            fmt = '%1.3f'
        elif dtick > 0.000099:
            fmt = '%1.4f'
        elif dtick > 0.0000099:
            fmt = '%1.5f'

        s =  fmt % val
        s.strip()
        s = s.replace('+', '')
        while s.find('e0')>0:
            s = s.replace('e0','e')
        while s.find('-0')>0:
            s = s.replace('-0','-')
        return s



#!/usr/bin/python
#
# simple example of MPlot

import wx
import wxmplot
import numpy

x   = numpy.linspace(0.0, 10.0, 201)
y1  = numpy.sin(3*x)/(x+2)
y2  = numpy.cos(2*x)*numpy.sin(2*x)
y3  = numpy.cos(2*x) + x/3
y4  = numpy.cos(2*x)*numpy.exp(-x/10.)
y5  = y4 + y2
y6  = 10*y1 +  y3
app = wx.App()

pframe = wxmplot.MultiPlotFrame(rows=2, cols=3, panelsize=(350, 275))


pframe.plot(x, y1, panel=(0, 0), labelfontsize=6)
pframe.plot(x, y2, panel=(0, 1), color='red',  labelfontsize=6)
pframe.plot(x, y3, panel=(0, 2), color='black', labelfontsize=5)
pframe.plot(x, y4, panel=(1, 0), fullbox=False)
pframe.plot(x, y5, panel=(1, 1), show_grid=False)
pframe.plot(x, y6, panel=(1, 2))
pframe.set_title('Test Plot', panel=(0, 0))
pframe.set_title('Plot2',panel=(1,0))
pframe.set_xlabel(r' $ R  (\AA) $ ')
pframe.Show()
pframe.Raise()

app.MainLoop()
